package main

import (
	"flag"
	"path/filepath"
	"time"

	"git.wntrmute.dev/kyle/nlink/nomad"

	"git.sr.ht/~thrrgilag/woodstock"
	"git.wntrmute.dev/kyle/goutils/config"
	"git.wntrmute.dev/kyle/goutils/die"
	log "git.wntrmute.dev/kyle/goutils/syslog"
)

var defaultRSS = "https://nomad.wntrmute.net/u/kyle.rss"

func defaultPath(file string) string {
	return filepath.Join("/etc", "nlink", file)
}

func fetchItems(db *nomad.DB, markOnly bool) error {
	rssFeed := nomad.NewURLSource(config.Get("local_rss_feed_url"))
	items, err := nomad.FetchRSS(rssFeed)
	if err != nil {
		return nil
	}

	items, err = db.Filter(nil, items)
	if err != nil {
		return err
	}

	if items == nil {
		return nil
	}

	if markOnly {
		tx, err := db.Begin()
		if err != nil {
			return err
		}

		for _, item := range items {
			if err = db.Mark(tx, item); err != nil {
				return err
			}
		}

		tx.Commit()

		if err := restoreDatabase(config.Get("local_database")); err != nil {
			return err
		}

		return nil
	}

	client := woodstock.NewClient(config.Get("pnut_id"), config.Get("pnut_secret"))
	client.SetAccessToken(config.Get("pnut_auth_token"))

	for _, item := range items {
		p := nomad.NewPost(item)
		if err = p.Fetch(); err != nil {
			log.Err(err)
			continue
		}

		if err = p.Post(client); err != nil {
			log.Err(err)
			continue
		}

		if err = db.Mark(nil, item); err != nil {
			log.Err(err)
			continue
		}
	}

	if err := saveDatabase(config.Get("local_database")); err != nil {
		log.Err(err)
	}

	return nil
}

const defaultInterval = 5 * time.Minute

func main() {
	var (
		configFile string
		interval   time.Duration
		initDB     bool
		level      string
		markOnly   bool
	)

	flag.StringVar(&configFile, "f", defaultPath("nlink.conf"), "`path` to config file")
	flag.BoolVar(&initDB, "i", false, "initialize a new DB")
	flag.StringVar(&level, "l", "DEBUG", "log level")
	flag.BoolVar(&markOnly, "m", false, "only mark posts as having been posted")
	flag.DurationVar(&interval, "t", defaultInterval, "interval between fetching posts")
	flag.Parse()

	logOpts := &log.Options{
		Level:       level,
		Tag:         "nlink",
		Facility:    "daemon",
		WriteSyslog: false,
	}

	if err := log.Setup(logOpts); err != nil {
		die.If(err)
	}

	log.Debugln("config keys")
	for _, k := range config.ListKeys() {
		log.Debugf("key: %s", k)
	}

	log.Debugf("loading config file %s\n", configFile)
	if err := config.LoadFile(configFile); err != nil {
		log.Fatal(err)
	}

	log.Infof("minio_bucket: %s", config.Get("minio_bucket"))
	if err := restoreDatabase(config.Get("local_database")); err != nil {
		log.Fatal(err)
	}

	if initDB {
		db := nomad.NewDB(config.Get("local_database"))
		if err := db.Create(); err != nil {
			log.Fatal(err)
		}
		db.Close()
	}

	log.Debugln("connecting to debug")
	db := nomad.NewDB(config.Get("local_database"))
	if err := db.Connect(); err != nil {
		log.Emerg(err)
	}

	defer db.Close()

	if markOnly {
		log.Info("marking database")
		if err := fetchItems(db, markOnly); err != nil {
			log.Fatal(err)
		}
		return
	}

	log.Debugf("will look for new items every %s\n", interval)
	for {
		log.Infoln("fetching items")
		if err := fetchItems(db, markOnly); err != nil {
			log.Info(err)
		}
		time.Sleep(interval)
	}
}
